@@ -1,5 +1,5 @@
 #![no_std]
-#![cfg_attr(docsrs, feature(doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_auto_cfg))]
 #![warn(missing_copy_implementations)]
 #![warn(missing_debug_implementations)]
 #![warn(missing_docs)]
@@ -8,6 +8,9 @@
 #![warn(unused_extern_crates)]
 #![warn(unused_import_braces)]
 #![warn(variant_size_differences)]
+// This would be forbid, except unsafe is necessary to work with `&mut str`,
+// nowhere else should use it
+#![deny(unsafe_code)]
 #![doc(test(attr(deny(warnings))))]
 
 //! Another [Base58][] codec implementation.
@@ -35,8 +38,10 @@
 //!  `std`   | **on**-by-default  | Implement [`Error`](std::error::Error) for error types
 //!  `alloc` | implied by `std`   | Support encoding/decoding to [`Vec`](alloc::vec::Vec) and [`String`](alloc::string::String) as appropriate
 //!  `check` | **off**-by-default | Integrated support for [Base58Check][]
+//!  `cb58`  | **off**-by-default | Integrated support for [CB58][]
 //!
 //! [Base58Check]: https://en.bitcoin.it/wiki/Base58Check_encoding
+//! [CB58]: https://support.avax.network/en/articles/4587395-what-is-cb58
 //!
 //! # Examples
 //!
@@ -66,8 +71,8 @@
 //!
 //! ```rust
 //! let (mut decoded, mut encoded) = ([0xFF; 8], String::with_capacity(10));
-//! bs58::decode("he11owor1d").into(&mut decoded)?;
-//! bs58::encode(decoded).into(&mut encoded)?;
+//! bs58::decode("he11owor1d").onto(&mut decoded)?;
+//! bs58::encode(decoded).onto(&mut encoded)?;
 //! assert_eq!("he11owor1d", encoded);
 //! # Ok::<(), Box<dyn std::error::Error>>(())
 //! ```
@@ -85,7 +90,7 @@ pub use alphabet::Alphabet;
 pub mod decode;
 pub mod encode;
 
-#[cfg(feature = "check")]
+#[cfg(any(feature = "check", feature = "cb58"))]
 const CHECKSUM_LEN: usize = 4;
 
 /// Possible check variants.
@@ -93,6 +98,8 @@ enum Check {
     Disabled,
     #[cfg(feature = "check")]
     Enabled(Option<u8>),
+    #[cfg(feature = "cb58")]
+    CB58(Option<u8>),
 }
 
 /// Setup decoder for the given string using the [default alphabet][Alphabet::DEFAULT].
@@ -123,7 +130,7 @@ enum Check {
 ///
 /// ```rust
 /// let mut output = [0xFF; 10];
-/// assert_eq!(8, bs58::decode("he11owor1d").into(&mut output)?);
+/// assert_eq!(8, bs58::decode("he11owor1d").onto(&mut output)?);
 /// assert_eq!(
 ///     [0x04, 0x30, 0x5e, 0x2b, 0x24, 0x73, 0xf0, 0x58, 0xFF, 0xFF],
 ///     output);
@@ -158,7 +165,7 @@ enum Check {
 /// let mut output = [0; 7];
 /// assert_eq!(
 ///     bs58::decode::Error::BufferTooSmall,
-///     bs58::decode("he11owor1d").into(&mut output).unwrap_err());
+///     bs58::decode("he11owor1d").onto(&mut output).unwrap_err());
 /// ```
 pub fn decode<I: AsRef<[u8]>>(input: I) -> decode::DecodeBuilder<'static, I> {
     decode::DecodeBuilder::from_input(input)
@@ -190,9 +197,9 @@ pub fn decode<I: AsRef<[u8]>>(input: I) -> decode::DecodeBuilder<'static, I> {
 ///
 /// ```rust
 /// let input = [0x04, 0x30, 0x5e, 0x2b, 0x24, 0x73, 0xf0, 0x58];
-/// let mut output = "goodbye world".to_owned();
-/// bs58::encode(input).into(&mut output)?;
-/// assert_eq!("he11owor1d", output);
+/// let mut output = "goodbye world ".to_owned();
+/// bs58::encode(input).onto(&mut output)?;
+/// assert_eq!("goodbye world he11owor1d", output);
 /// # Ok::<(), bs58::encode::Error>(())
 /// ```
 ///
@@ -207,7 +214,7 @@ pub fn decode<I: AsRef<[u8]>>(input: I) -> decode::DecodeBuilder<'static, I> {
 /// let mut output = [0; 7];
 /// assert_eq!(
 ///     bs58::encode::Error::BufferTooSmall,
-///     bs58::encode(input).into(&mut output[..]).unwrap_err());
+///     bs58::encode(input).onto(&mut output[..]).unwrap_err());
 /// ```
 pub fn encode<I: AsRef<[u8]>>(input: I) -> encode::EncodeBuilder<'static, I> {
     encode::EncodeBuilder::from_input(input)
